@@ -4,11 +4,13 @@ import collections
 from functools import partial
 from Queue import Full, Empty
 
+import tornado
 from tornado import ioloop
+from tornado import gen
 from tornado.concurrent import Future
 
 
-version_tuple = (0, 8)
+version_tuple = (1, 0, 1)
 
 version = '.'.join(map(str, version_tuple))
 """Current version of Toro."""
@@ -79,6 +81,16 @@ class _TimeoutFuture(Future):
             self._timeout_handle = None
 
 
+class _ContextManagerList(list):
+    def __enter__(self, *args, **kwargs):
+        for obj in self:
+            obj.__enter__(*args, **kwargs)
+
+    def __exit__(self, *args, **kwargs):
+        for obj in self:
+            obj.__exit__(*args, **kwargs)
+
+
 class _ContextManagerFuture(Future):
     """A Future that can be used with the "with" statement.
 
@@ -89,7 +101,7 @@ class _ContextManagerFuture(Future):
             pass
 
     At the end of the block, the Future's exit callback is run. Used for
-    Lock.acquire() and Semaphore.acquire().
+    Lock.acquire, Semaphore.acquire, RWLock.acquire_read / acquire_write.
     """
     def __init__(self, wrapped, exit_callback):
         super(_ContextManagerFuture, self).__init__()
@@ -351,6 +363,7 @@ class Queue(object):
         self.putters = collections.deque([])
         self._init(maxsize)
 
+    # These three are overridable in subclasses.
     def _init(self, maxsize):
         self.queue = collections.deque()
 
@@ -825,3 +838,130 @@ class Lock(object):
             " 'with lock'")
 
     __exit__ = __enter__
+
+
+if tornado.version_info[:2] >= (4, 2):
+    tornado_multi_future = gen.multi_future
+else:
+    tornado_multi_future = lambda futures, quiet_exceptions: futures
+
+
+class RWLock(object):
+    """A reader-writer lock for coroutines.
+
+    It is created unlocked. When unlocked, :meth:`acquire_write` always changes
+    the state to locked. When unlocked, :meth:`acquire_read` can changed the
+    state to locked, if :meth:`acquire_read` was called max_readers times. When
+    the state is locked, yielding :meth:`acquire_read`/meth:`acquire_write`
+    waits until a call to :meth:`release_write` in case of locking on write, or
+    :meth:`release_read` in case of locking on read.
+
+    The :meth:`release_read` method should only be called in the locked-on-read
+    state; an attempt to release an unlocked lock raises RuntimeError.
+
+    The :meth:`release_write` method should only be called in the locked on
+    write state; an attempt to release an unlocked lock raises RuntimeError.
+
+    When more than one coroutine is waiting for the lock, the first one
+    registered is awakened by :meth:`release_read`/:meth:`release_write`.
+
+    :meth:`acquire_read`/:meth:`acquire_write` support the context manager
+    protocol:
+
+    >>> from tornado import gen
+    >>> import toro
+    >>> lock = toro.RWLock(max_readers=10)
+    >>>
+    >>> @gen.coroutine
+    ... def f():
+    ...    with (yield lock.acquire_read()):
+    ...        assert not lock.locked()
+    ...
+    ...    with (yield lock.acquire_write()):
+    ...        assert lock.locked()
+    ...
+    ...    assert not lock.locked()
+
+    :Parameters:
+      - `max_readers`: Optional max readers value, default 1.
+      - `io_loop`: Optional custom IOLoop.
+    """
+    def __init__(self, max_readers=1, io_loop=None):
+        self._max_readers = max_readers
+        self._block = BoundedSemaphore(value=max_readers, io_loop=io_loop)
+
+    def __str__(self):
+        return "<%s _block=%s>" % (
+            self.__class__.__name__,
+            self._block)
+
+    def acquire_read(self, deadline=None):
+        """Attempt to lock for read. Returns a Future.
+
+        The Future raises :exc:`toro.Timeout` if the deadline passes.
+
+        :Parameters:
+          - `deadline`: Optional timeout, either an absolute timestamp
+            (as returned by ``io_loop.time()``) or a ``datetime.timedelta`` for
+            a deadline relative to the current time.
+        """
+        return self._block.acquire(deadline)
+
+    @gen.coroutine
+    def acquire_write(self, deadline=None):
+        """Attempt to lock for write. Returns a Future.
+
+        The Future raises :exc:`toro.Timeout` if the deadline passes.
+
+        :Parameters:
+          - `deadline`: Optional timeout, either an absolute timestamp
+            (as returned by ``io_loop.time()``) or a ``datetime.timedelta`` for
+            a deadline relative to the current time.
+        """
+        futures = [self._block.acquire(deadline) for _ in
+                   xrange(self._max_readers)]
+        try:
+            managers = yield tornado_multi_future(futures,
+                                                  quiet_exceptions=Timeout)
+        except Timeout:
+            for f in futures:
+                # Avoid traceback logging.
+                f.exception()
+            raise
+
+        raise gen.Return(_ContextManagerList(managers))
+
+    def release_read(self):
+        """Releases one reader.
+
+        If any coroutines are waiting for :meth:`acquire_read` (in case of full
+        readers queue), the first in line is awakened.
+
+        If not locked, raise a RuntimeError.
+        """
+        if self._block.counter == self._max_readers:
+            raise RuntimeError('release unlocked lock')
+        self._block.release()
+
+    def release_write(self):
+        """Releases after write.
+
+        The first in queue will be awakened after release.
+
+        If not locked, raise a RuntimeError.
+        """
+        if not self.locked():
+            raise RuntimeError('release unlocked lock')
+        for i in xrange(self._max_readers):
+            self._block.release()
+
+    def locked(self):
+        """``True`` if the lock has been acquired"""
+        return self._block.locked()
+
+    def __enter__(self):
+        raise RuntimeError(
+            "Use RWLock like 'with (yield lock)', not like"
+            " 'with lock'")
+
+    __exit__ = __enter__
